import torch
import torch.nn as nn
import torch.nn.functional as F

class EFRCELoss(nn.Module):
    def __init__(self, temperature_scale=1.0, fuzzy_scale=1.0):
        super(EFRCELoss, self).__init__()
        self.criterion = torch.nn.CrossEntropyLoss()
        self.temperature_scale = temperature_scale
        self.fuzzy_scale = fuzzy_scale

    def forward(self, outputs, ys):
        outputs = outputs/self.temperature_scale
        ys_onehot = F.one_hot(ys, outputs.shape[1])
        outputs[~(ys_onehot.bool())] = outputs[~(ys_onehot.bool())]*self.fuzzy_scale
        loss = self.criterion(outputs, ys)
        num_class = outputs.shape[1]
        for i in range(num_class):
            ys_temp = torch.zeros_like(ys)+i
            loss -= self.criterion(outputs, ys_temp)/num_class

        return loss
